<?php

namespace Drupal\commerceg_product_group\Configure;

use Drupal\commerceg\MachineName\Plugin\GroupRelationType;
use Drupal\commerceg_product_group\MachineName\Bundle\Group as GroupBundle;
use Drupal\commerceg_product_group\MachineName\ConfigEntity\GroupRole;
use Drupal\commerceg_product_group\MachineName\Plugin\GroupRelationType as ProductGroupGroupRelationType;
use Drupal\group\PermissionScopeInterface;

use Drupal\commerce_product\Entity\ProductTypeInterface;
use Drupal\group\Entity\GroupTypeInterface;

use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Default implementation of the Product Group Installer.
 *
 * Provides certain default configuration that makes sense for most
 * scenarios. Applications can override it to provide its own configuration
 * required.
 */
class Installer implements InstallerInterface {

  /**
   * The group relationship storage.
   *
   * @var \Drupal\group\Entity\Storage\GroupRelationshipStorageInterface
   */
  protected $relationshipStorage;

  /**
   * The group relationship type storage.
   *
   * @var \Drupal\group\Entity\Storage\GroupRelationshipTypeStorageInterface
   */
  protected $relationshipTypeStorage;

  /**
   * The group role storage.
   *
   * @var \Drupal\group\Entity\Storage\GroupRoleStorageInterface
   */
  protected $groupRoleStorage;

  /**
   * The group type storage.
   *
   * @var \Drupal\Core\Config\Entity\ConfigEntityStorageInterface
   */
  protected $groupTypeStorage;

  /**
   * Constructs a new Installer object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->groupRoleStorage = $entity_type_manager->getStorage('group_role');
    $this->groupTypeStorage = $entity_type_manager->getStorage('group_type');

    $this->relationshipStorage = $entity_type_manager
      ->getStorage('group_relationship');
    $this->relationshipTypeStorage = $entity_type_manager
      ->getStorage('group_relationship_type');
  }

  /**
   * {@inheritdoc}
   */
  public function install() {
    $this->installGroupType();
    $this->installDefaultRoles();
  }

  /**
   * {@inheritdoc}
   */
  public function installGroupType() {
    // Do nothing if the group type already exists. This can happen if the
    // module is uninstalled and then installed again.
    if ($this->isGroupTypeInstalled()) {
      return;
    }

    $group_type = $this->groupTypeStorage->create([
      'id' => GroupBundle::PRODUCTS,
      'label' => 'Products',
      'description' => 'A group of products that can be used to restrict manager or customer access.',
    ]);
    $group_type->save();
  }

  /**
   * {@inheritdoc}
   */
  public function installDefaultRoles() {
    foreach ($this->getDefaultRoles() as $id => $role) {
      $this->installRole($id, $role['label']);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function installProductType(ProductTypeInterface $product_type) {
    if ($this->isProductTypeInstalled($product_type)) {
      return;
    }

    $group_type = $this->groupTypeStorage->load(GroupBundle::PRODUCTS);
    $this->relationshipTypeStorage
      ->createFromPlugin(
        $group_type,
        GroupRelationType::PRODUCT . ':' . $product_type->id()
      )
      ->save();
  }

  /**
   * {@inheritdoc}
   */
  public function uninstallProductType(ProductTypeInterface $product_type) {
    $relationship_type = $this->getProductTypeRelationshipType($product_type);
    $this->relationshipTypeStorage->delete([$relationship_type]);
  }

  /**
   * {@inheritdoc}
   */
  public function isProductTypeInstalled(ProductTypeInterface $product_type) {
    return $this->getProductTypeRelationshipType($product_type) ? TRUE : FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function canUninstallProductType(ProductTypeInterface $product_type) {
    $relationship_type = $this->getProductTypeRelationshipType($product_type);

    // We cannot uninstall the group relationship type if there is group
    // relationship for it.
    $count = $this->relationshipStorage
      ->getQuery()
      ->accessCheck(FALSE)
      ->condition('type', $relationship_type->id())
      ->count()
      ->execute();

    return $count ? FALSE : TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function installManagedMembershipRoles() {
    foreach ($this->getManagedMembershipRoles() as $id => $role) {
      $this->installRole($id, $role['label']);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function installProductGroupRelationshipType(
    GroupTypeInterface $group_type
  ) {
    if ($this->isProductGroupRelationshipTypeInstalled($group_type)) {
      return;
    }

    $this->relationshipTypeStorage
      ->createFromPlugin(
        $group_type,
        ProductGroupGroupRelationType::PRODUCTS
      )
      ->save();
  }

  /**
   * Returns whether the Products group type is installed or not.
   *
   * @return bool
   *   TRUE if the Products group type exists, FALSE otherwise.
   */
  protected function isGroupTypeInstalled() {
    $group_type = $this->groupTypeStorage->load(GroupBundle::PRODUCTS);
    return $group_type ? TRUE : FALSE;
  }

  /**
   * Installs the role with the given ID and label.
   *
   * @param string $id
   *   The ID of the group role to create, including the group type prefix
   *   e.g. `commerceg_products-customer`.
   * @param string $label
   *   The label of the group role. It will be used only if the group role is
   *   created i.e. if the role already exists and it has a different label, it
   *   will not be corrected.
   */
  protected function installRole($id, $label) {
    if ($this->isRoleInstalled($id)) {
      return;
    }

    $role = $this->groupRoleStorage->create([
      'id' => $id,
      'label' => $label,
      'group_type' => GroupBundle::PRODUCTS,
      'scope' => PermissionScopeInterface::INDIVIDUAL_ID,
    ]);
    $role->save();
  }

  /**
   * Returns whether the group role with the given ID exists or not.
   *
   * @param string $id
   *   The ID of the group role to check, including the group type prefix
   *   e.g. `commerceg_products-customer`.
   *
   * @return bool
   *   TRUE if the group role exists, FALSE otherwise.
   */
  protected function isRoleInstalled($id) {
    return $this->groupRoleStorage->load($id) ? TRUE : FALSE;
  }

  /**
   * Returns the default group roles required for general functionality.
   *
   * @return array
   *   An associative array keyed by the role ID and containing an associative
   *   array with further information about the role. Supported information is:
   *   - id: The ID (machine name) of the role.
   *   - label: The label (human-friendly name) of the role.
   */
  protected function getDefaultRoles() {
    return [
      GroupRole::CUSTOMER => [
        'id' => GroupRole::CUSTOMER,
        'label' => 'Customer',
      ],
      GroupRole::MANAGER => [
        'id' => GroupRole::MANAGER,
        'label' => 'Manager',
      ],
    ];
  }

  /**
   * Returns the relationship type for the given product type.
   *
   * @param Drupal\commerce_product\Entity\ProductTypeInterface $product_type
   *   The product type.
   *
   * @return \Drupal\group\Entity\GroupRelationshipInterface|null
   *   The group relationship type, or NULL if the group relation type plugin
   *   for the given product type on the Products group is not installed.
   */
  protected function getProductTypeRelationshipType(
    ProductTypeInterface $product_type
  ) {
    $relationship_types = $this->relationshipTypeStorage->loadByPluginId(
      GroupRelationType::PRODUCT . ':' . $product_type->id()
    );
    foreach ($relationship_types as $relationship_type) {
      if ($relationship_type->getGroupType()->id() !== GroupBundle::PRODUCTS) {
        continue;
      }

      return $relationship_type;
    }
  }

  /**
   * Returns the group roles required for the managed memberships functionality.
   *
   * @return array
   *   An associative array keyed by the role ID and containing an associative
   *   array with further information about the role. Supported information is:
   *   - id: The ID (machine name) of the role.
   *   - label: The label (human-friendly name) of the role.
   */
  protected function getManagedMembershipRoles() {
    return [
      GroupRole::GROUP_CUSTOMER => [
        'id' => GroupRole::GROUP_CUSTOMER,
        'label' => 'Group customer',
      ],
    ];
  }

  /**
   * Returns whether the relationship type for the given group type exists.
   *
   * More specifically, it checks for the group relationship type that
   * associates product groups with groups of the given type.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $group_type
   *   The group type for which to check whether the group relationship type
   *   exists.
   */
  protected function isProductGroupRelationshipTypeInstalled(
    GroupTypeInterface $group_type
  ) {
    return $this->getProductGroupRelationshipType($group_type) ? TRUE : FALSE;
  }

  /**
   * Returns the relationship type for the given type.
   *
   * More specifically, it returns the group relationship type that associates
   * product groups with groups of the given type.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $group_type
   *   The group type for which to return the group relationship type.
   *
   * @return \Drupal\group\Entity\GroupRelationshipInterface|null
   *   The group relationship type, or NULL if the product group is not
   *   installed on the given group type.
   */
  protected function getProductGroupRelationshipType(
    GroupTypeInterface $group_type
  ) {
    $relationship_types = $this->relationshipTypeStorage->loadByPluginId(
      ProductGroupGroupRelationType::PRODUCTS
    );
    foreach ($relationship_types as $relationship_type) {
      if ($relationship_type->getGroupType()->id() !== $group_type->id()) {
        continue;
      }

      return $relationship_type;
    }
  }

}
