<?php

namespace Drupal\commerceg_product_group\Membership;

use Drupal\commerceg\MachineName\Plugin\GroupRelationType;
use Drupal\commerceg_product_group\MachineName\Bundle\Group as GroupBundle;
use Drupal\commerceg_product_group\MachineName\ConfigEntity\GroupRole;
use Drupal\commerceg_product_group\MachineName\Plugin\GroupRelationType as ProductGroupGroupRelationType;

use Drupal\group\Entity\GroupInterface;
use Drupal\group\Entity\GroupRelationshipInterface;
use Drupal\group\Entity\GroupRelationshipTypeInterface;
use Drupal\group\Entity\GroupTypeInterface;
use Drupal\group\GroupMembershipLoaderInterface;
use Drupal\user\UserInterface;

use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Default implementation of the Membership Manager.
 */
class Manager implements ManagerInterface {

  /**
   * The membership loader.
   *
   * @var \Drupal\group\GroupMembershipLoaderInterface
   */
  protected $membershipLoader;

  /**
   * The group relationship storage.
   *
   * @var \Drupal\group\Entity\Storage\GroupRelationshipStorageInterface
   */
  protected $relationshipStorage;

  /**
   * The group relationship type storage.
   *
   * @var \Drupal\group\Entity\Storage\GroupRelationshipTypeStorageInterface
   */
  protected $relationshipTypeStorage;

  /**
   * The Group Customer role entity.
   *
   * @var \Drupal\group\Entity\GroupRoleInterface
   */
  protected $customerRole;

  /**
   * Constructs a new Manager object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\group\GroupMembershipLoaderInterface $membership_loader
   *   The group membership loader.
   *
   * @throws \InvalidArgumentException
   *   If the group customer role used to give access to users in product groups
   *   via membership in other groups does not exist.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    GroupMembershipLoaderInterface $membership_loader
  ) {
    $this->membershipLoader = $membership_loader;

    $this->relationshipStorage = $entity_type_manager
      ->getStorage('group_relationship');
    $this->relationshipTypeStorage = $entity_type_manager
      ->getStorage('group_relationship_type');

    $this->customerRole = $entity_type_manager
      ->getStorage('group_role')
      ->load(GroupRole::GROUP_CUSTOMER);

    // The Group Customer role is required for the Managed Memberships feature
    // to work. It is installed upon module installation; if somehow it was
    // removed, something is wrong.
    if (!$this->customerRole) {
      throw new \RuntimeException(
        'The group customer group role does not exist on the %s group type.',
        GroupBundle::PRODUCTS
      );
    }
  }

  /**
   * {@inheritdoc}
   */
  public function ensureMembershipsForAssociatedGroup(
    GroupInterface $product_group,
    GroupInterface $associated_group
  ) {
    if ($product_group->bundle() !== GroupBundle::PRODUCTS) {
      throw new \InvalidArgumentException(
        'The group that users will be added to must be of %s group type, %s given.',
        GroupBundle::PRODUCTS,
        $product_group->bundle()
      );
    }

    $memberships = $this->membershipLoader->loadByGroup($associated_group);
    foreach ($memberships as $membership) {
      $this->ensureMembership($membership->getUser(), $product_group);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function ensureMembershipsForUser(
    UserInterface $user,
    GroupInterface $associated_group
  ) {
    // Add memberships of the user in all product groups associated
    // with the given group.
    $product_groups = $this->getAssociatedProductGroups($associated_group);
    foreach ($product_groups as $product_group) {
      $this->ensureMembership(
        $user,
        $product_group
      );
    };
  }

  /**
   * {@inheritdoc}
   */
  public function removeMembershipsForAssociatedGroup(
    GroupInterface $product_group,
    GroupInterface $associated_group
  ) {
    if ($product_group->bundle() !== GroupBundle::PRODUCTS) {
      throw new \InvalidArgumentException(
        'The group that users will be added to must be of %s group type, %s given.',
        GroupBundle::PRODUCTS,
        $product_group->bundle()
      );
    }

    $memberships = $this->membershipLoader->loadByGroup($associated_group);
    foreach ($memberships as $membership) {
      $this->removeMembership(
        $membership->getUser(),
        $product_group,
        $associated_group
      );
    }
  }

  /**
   * {@inheritdoc}
   */
  public function removeMembershipsForUser(
    UserInterface $user,
    GroupInterface $associated_group
  ) {
    // Remove any memberships of the user from all product groups associated
    // with the given group.
    $product_groups = $this->getAssociatedProductGroups($associated_group);
    foreach ($product_groups as $product_group) {
      $this->removeMembership(
        $user,
        $product_group,
        $associated_group
      );
    };
  }

  /**
   * {@inheritdoc}
   */
  public function ensureMembership(
    UserInterface $user,
    GroupInterface $product_group
  ) {
    $relationship_type = $this->getRelationshipType(
      $product_group->getGroupType(),
      GroupRelationType::MEMBERSHIP
    );
    if (!$relationship_type) {
      throw new \Exception(sprintf(
        'Cannot add the user with ID %s to the product group with ID %s because the %s group relation type plugin is not installed on the group type.',
        $user->id(),
        $group->id(),
        GroupRelationType::MEMBERSHIP
      ));
    }

    $relationship = $this->getProductGroupMembershipRelationship(
      $user,
      $product_group,
      $relationship_type,
      TRUE
    );
    if (!$relationship) {
      return;
    }

    $this->ensureRole($relationship);
  }

  /**
   * {@inheritdoc}
   */
  public function removeMembership(
    UserInterface $user,
    GroupInterface $product_group,
    GroupInterface $associated_group
  ) {
    $relationship_type = $this->getRelationshipType(
      $product_group->getGroupType(),
      GroupRelationType::MEMBERSHIP
    );
    if (!$relationship_type) {
      throw new \Exception(sprintf(
        'Cannot add the user with ID %s to the product group with ID %s because the %s group relation type plugin is not installed on the group type.',
        $user->id(),
        $group->id(),
        GroupRelationType::MEMBERSHIP
      ));
    }

    $relationship = $this->getProductGroupMembershipRelationship(
      $user,
      $product_group,
      $relationship_type,
      FALSE
    );
    if (!$relationship) {
      return;
    }

    $has_other_memberships = $this->hasOtherAssociatedGroupMemberships(
      $user,
      $product_group,
      $associated_group
    );
    if ($has_other_memberships) {
      return;
    }

    $this->removeRole($relationship);

  }

  /**
   * Returns whether the user has memberships in other associated groups.
   *
   * When removing a product group membership because the association between
   * the product group and an associated group has been removed, we do not want
   * to remove the membership if the user has a membership in one or more
   * other associated groups.
   *
   * This method returns whether the user has at least one membership on another
   * group other than the given associated group but of the same type, that is
   * also associated with the given product group.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user to check the membership for.
   * @param \Drupal\group\Entity\GroupInterface $product_group
   *   The product group.
   * @param \Drupal\group\Entity\GroupInterface $associated_group
   *   The associated group to exclude when looking for existing memberships.
   *
   * @I Maybe it's more optimal to search for user-to-parent relationships
   *    first; a user would more commonly belong to one or few parent groups,
   *    while a product group might be added to plenty of parent groups.
   */
  protected function hasOtherAssociatedGroupMemberships(
    UserInterface $user,
    GroupInterface $product_group,
    GroupInterface $associated_group
  ) {
    $association_relationship_type = $this->getRelationshipType(
      $associated_group->getGroupType(),
      ProductGroupGroupRelationType::PRODUCTS
    );
    if (!$association_relationship_type) {
      throw new \RuntimeException(
        'The %s group relation type plugin is not installed on the %s group type.',
        ProductGroupGroupRelationType::PRODUCTS,
        $associated_group->bundle()
      );
    }

    // Get group relationships of the product group to other associated groups
    // i.e. excluding the given associated group.
    $association_relationship_ids = $this->relationshipStorage
      ->getQuery()
      ->accessCheck(FALSE)
      ->condition('type', $association_relationship_type->id())
      ->condition('entity_id', $product_group->id())
      ->condition('gid', $associated_group->id(), '!=')
      ->execute();
    if (!$association_relationship_ids) {
      return FALSE;
    }

    $membership_relationship_type = $this->getRelationshipType(
      $associated_group->getGroupType(),
      GroupRelationType::MEMBERSHIP
    );
    $association_relationships = $this->relationshipStorage->loadMultiple(
      $association_relationship_ids
    );

    // Check if the user has membership to at least one other associated group.
    foreach ($association_relationships as $relationship) {
      $count = $this->relationshipStorage
        ->getQuery()
        ->accessCheck(FALSE)
        ->condition('type', $membership_relationship_type->id())
        ->condition('entity_id', $user->id())
        ->condition('gid', $relationship->getGroup()->id())
        ->count()
        ->execute();
      if ($count) {
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * Returns the group relationship type for the given group type and plugin ID.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $group_type
   *   The group type to get the relationship type for.
   * @param string $plugin_id
   *   The plugin ID to get the relationship type for.
   *
   * @return \Drupal\group\Entity\GroupRelationshipTypeInterface|null
   *   The group relationship type for the given group type and plugin, or NULL
   *   if none was found.
   */
  protected function getRelationshipType(
    GroupTypeInterface $group_type,
    $plugin_id
  ) {
    $relationship_types = $this->relationshipTypeStorage
      ->loadByGroupType($group_type);
    foreach ($relationship_types as $relationship_type) {
      if ($relationship_type->getPluginId() !== $plugin_id) {
        continue;
      }

      return $relationship_type;
    }
  }

  /**
   * Returns the membership group relationship for the given user/product group.
   *
   * Optionally, create a group relationship if it doesn't exist. Note that the
   * group relationship entity is returned unsaved.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user.
   * @param \Drupal\group\Entity\GroupInterface $group
   *   The product group.
   * @param \Drupal\group\Entity\GroupRelationshipTypeInterface $relationship_type
   *   The group relationship type that associates products groups with users.
   * @param bool $create
   *   TRUE to create a group relationship if it doesn't exist i.e. associate the
   *   user with the product group, FALSE otherwise.
   *
   * @return \Drupal\group\Entity\GroupRelationshipInterface|null
   *   The group relationship that associates the given user with the given
   *   product group, or NULL if it doesn't exist and `$create` is set to FALSE.
   *
   * @throws \RuntimeException
   *   If more than one existing group relationship was found.
   */
  protected function getProductGroupMembershipRelationship(
    UserInterface $user,
    GroupInterface $group,
    GroupRelationshipTypeInterface $relationship_type,
    $create
  ) {
    $properties = [
      'type' => $relationship_type->id(),
      'entity_id' => $user->id(),
      'gid' => $group->id(),
    ];
    $relationships = $this->relationshipStorage->loadByProperties($properties);

    if (!$relationships && $create) {
      return $this->relationshipStorage->create($properties);
    }

    switch (count($relationships)) {
      case 1:
        return current($relationships);

      case 0:
        return;

      default:
        throw new \RuntimeException(
          'More than one group relationship entity found for user with ID %s and product group with ID %s',
          $user->id(),
          $group->id()
        );
    }
  }

  /**
   * Returns all product groups associated with the given group.
   *
   * @param \Drupal\group\Entity\GroupInterface $associated_group
   *   The group to return the product groups for.
   *
   * @return \Drupal\group\Entity\GroupInterface[]
   *   The associated product groups.
   */
  protected function getAssociatedProductGroups(
    GroupInterface $associated_group
  ) {
    $association_relationship_type = $this->getRelationshipType(
      $associated_group->getGroupType(),
      ProductGroupGroupRelationType::PRODUCTS
    );
    if (!$association_relationship_type) {
      throw new \RuntimeException(
        'The %s group relation type plugin is not installed on the %s group type.',
        ProductGroupGroupRelationType::PRODUCTS,
        $associated_group->bundle()
      );
    }

    // Get all group relationships of product groups to the given associated
    // group.
    $association_relationship_ids = $this->relationshipStorage
      ->getQuery()
      ->accessCheck(FALSE)
      ->condition('type', $association_relationship_type->id())
      ->condition('gid', $associated_group->id())
      ->execute();
    if (!$association_relationship_ids) {
      return [];
    }

    return array_map(
      function ($relationship) {
        return $relationship->getEntity();
      },
      $this->relationshipStorage->loadMultiple($association_relationship_ids)
    );
  }

  /**
   * Ensures that the given relationship has the Group Customer group role.
   *
   * @param \Drupal\group\Entity\GroupRelationshipInterface $relationship
   *   The relationship entity that associates a user with a product group.
   *
   * @throws \RuntimeException
   *   If the given group relationship does not belong to a Products group.
   * @throws \RuntimeException
   *   If the given group relationship is not of Group Membership plugin.
   */
  protected function ensureRole(GroupRelationshipInterface $relationship) {
    $bundle = $relationship->getGroup()->bundle();
    if ($bundle !== GroupBundle::PRODUCTS) {
      throw new \RuntimeException(sprintf(
        'Group relationship should belong to a group of %s type, %s given.',
        GroupBundle::PRODUCTS,
        $bundle
      ));
    }

    $plugin_id = $relationship->getPlugin()->getPluginId();
    if ($plugin_id !== GroupRelationType::MEMBERSHIP) {
      throw new \RuntimeException(sprintf(
        'Group relationship of %s plugin expected, %s given.',
        GroupRelationType::MEMBERSHIP,
        $plugin_id
      ));
    }

    $field_value = $relationship->get('group_roles')->getValue();
    // We do nothing if the role is referenced already.
    foreach ($field_value as $field_item_value) {
      if ($field_item_value['target_id'] === $this->customerRole->id()) {
        return;
      }
    }

    $field_value[] = ['target_id' => $this->customerRole->id()];
    $relationship->set('group_roles', $field_value);
    $this->relationshipStorage->save($relationship);
  }

  /**
   * Removes the Group Customer group role from the give group relationship.
   *
   * @param \Drupal\group\Entity\GroupRelationshipInterface $relationship
   *   The group relationship entity that associates a user with a product group.
   *
   * @throws \RuntimeException
   *   If the given group relationship does not belong to a Products group.
   * @throws \RuntimeException
   *   If the given group relationship is not of Group Membership plugin.
   */
  protected function removeRole(GroupRelationshipInterface $relationship) {
    $bundle = $relationship->getGroup()->bundle();
    if ($bundle !== GroupBundle::PRODUCTS) {
      throw new \RuntimeException(sprintf(
        'Group relationship should belong to a group of %s type, %s given.',
        GroupBundle::PRODUCTS,
        $bundle
      ));
    }

    $plugin_id = $relationship->getPlugin()->getPluginId();
    if ($plugin_id !== GroupRelationType::MEMBERSHIP) {
      throw new \RuntimeException(sprintf(
        'Group relationship of %s plugin expected, %s given.',
        GroupRelationType::MEMBERSHIP,
        $plugin_id
      ));
    }

    $changed = FALSE;
    $field_value = $relationship->get('group_roles')->getValue();
    foreach ($field_value as $delta => $field_item_value) {
      if ($field_item_value['target_id'] === $this->customerRole->id()) {
        $changed = TRUE;
        unset($field_value[$delta]);
      }
    }

    if (!$changed) {
      return;
    }

    // If the user still has other roles, they must have a non-managed
    // membership. We keep the membership.
    if (count($field_value)) {
      $relationship->set('group_roles', $field_value);
      $this->relationshipStorage->save($relationship);
    }
    // Otherwise, the membership is removed.
    else {
      $this->relationshipStorage->delete([$relationship]);
    }
  }

}
