<?php

namespace Drupal\commerceg_product_group\Plugin\Group\Relation;

use Drupal\commerceg_product_group\MachineName\Bundle\Group as GroupBundle;
use Drupal\group\Plugin\Group\Relation\GroupRelationBase;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslationInterface;

use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a relation type for Products groups.
 *
 * Entity access is disabled because we simply want to associate product groups
 * with other groups. Access control happens with the managed memberships
 * feature and the role/permissions configured on product groups.
 *
 * @GroupRelationType(
 *   id = "commerceg_products",
 *   label = @Translation("Group product group"),
 *   description = @Translation("Adds product groups to groups."),
 *   entity_type_id = "group",
 *   entity_bundle = "commerceg_products",
 *   entity_access = FALSE,
 *   reference_label = @Translation("Name"),
 *   reference_description = @Translation("The name of the product group to add to the group")
 * )
 */
class Products extends GroupRelationBase implements
  ContainerFactoryPluginInterface {

  /**
   * The current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $account;

  /**
   * Constructs a new Products object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current user account.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    TranslationInterface $string_translation,
    AccountInterface $account
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    // @I The current user account seems unused, remove it.
    $this->account = $account;

    // Injections required by traits.
    $this->stringTranslation = $string_translation;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('string_translation'),
      $container->get('current_user')->getAccount()
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $config = parent::defaultConfiguration();
    $config['entity_cardinality'] = 1;

    return $config;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(
    array $form,
    FormStateInterface $form_state
  ) {
    $form = parent::buildConfigurationForm($form, $form_state);

    // Disable the entity cardinality field as the functionality of this module
    // relies on a cardinality of 1. We don't just hide it, though, to keep a UI
    // that's consistent with other group relations.
    $form['entity_cardinality']['#disabled'] = TRUE;
    $form['entity_cardinality']['#description'] .= '<br /><em>' . $this->t(
      "This field has been disabled by the plugin to guarantee the functionality
       that's expected of it."
    ) . '</em>';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $dependencies = parent::calculateDependencies();
    $dependencies['config'][] = 'group.type.' . GroupBundle::PRODUCTS;

    return $dependencies;
  }

}
